@@ -1,4 +1,4 @@
-# Copyright 2015 Jeffrey Kegler
+# Copyright 2013 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use strict;
 use warnings;
 
 use vars qw( $VERSION $STRING_VERSION );
-$VERSION        = '2.105_000';
+$VERSION        = '2.062000';
 $STRING_VERSION = $VERSION;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -35,7 +35,6 @@ package Marpa::R2::HTML::Internal;
 # Data::Dumper is used in tracing
 use Data::Dumper;
 
-use Marpa::R2::HTML::Internal;
 use Marpa::R2::HTML::Config;
 use Carp ();
 use HTML::Parser 3.69;
@@ -59,6 +58,18 @@ use Marpa::R2::Thin::Trace;
 
 # constants
 
+BEGIN {
+    my $structure = <<'END_OF_STRUCTURE';
+    :package=Marpa::R2::HTML::Internal::TDesc
+    TYPE
+    START_TOKEN
+    END_TOKEN
+    VALUE
+    RULE_ID
+END_OF_STRUCTURE
+    Marpa::R2::offset($structure);
+} ## end BEGIN
+
 use constant PHYSICAL_TOKEN      => 42;
 use constant RUBY_SLIPPERS_TOKEN => 43;
 
@@ -77,6 +88,23 @@ ERROR: for my $error_number ( 0 .. $#LIBMARPA_ERROR_NAMES ) {
     }
 } ## end ERROR: for my $error_number ( 0 .. $#LIBMARPA_ERROR_NAMES )
 
+BEGIN {
+    my $structure = <<'END_OF_STRUCTURE';
+    :package=Marpa::R2::HTML::Internal::Token
+    TOKEN_ID
+    =TAG_NAME
+    TYPE
+    LINE
+    COL
+    =COLUMN
+    START_OFFSET
+    END_OFFSET
+    IS_CDATA
+    ATTR
+END_OF_STRUCTURE
+    Marpa::R2::offset($structure);
+} ## end BEGIN
+
 use Marpa::R2::HTML::Callback;
 {
     my $submodule_version = $Marpa::R2::HTML::Callback::VERSION;
@@ -453,7 +481,7 @@ sub parse {
 
     my ($core_rules,   $runtime_tag,
         $rank_by_name, $is_empty_element,
-        $primary_group_by_tag
+	$primary_group_by_tag
     ) = $self->{config}->contents();
     $self->{is_empty_element} = $is_empty_element;
     if ($self->{dump_config}) {
@@ -470,7 +498,7 @@ sub parse {
         my $rhs    = $rule->{rhs};
         my $min    = $rule->{min};
         my $action = $rule->{action};
-        my @symbol_ids = ();
+	my @symbol_ids = ();
         for my $symbol_name ( $lhs, @{$rhs} ) {
             push @symbol_ids,
                 $tracer->symbol_by_name($symbol_name)
@@ -532,7 +560,7 @@ sub parse {
         PROCESS_TOKEN_TYPE: {
             if ($is_cdata) {
                 $raw_token->[Marpa::R2::HTML::Internal::Token::TOKEN_ID] =
-                    $SYMID_CDATA;
+		    $SYMID_CDATA;
                 last PROCESS_TOKEN_TYPE;
             }
             if ( $token_type eq 'T' ) {
@@ -711,7 +739,7 @@ sub parse {
                 next SYMBOL;
             } ## end if ( not defined $placement )
             my $tag = substr $rejected_symbol_name, 2;
-            my $primary_group = $primary_group_by_tag->{$tag};
+	    my $primary_group = $primary_group_by_tag->{$tag};
             my $element_type = defined $primary_group ? (substr $primary_group, 4) : 'anywhere';
             $ruby_vector =
                 $ruby_vectors{ q{<} . $placement . q{%} . $element_type . q{>} };
@@ -732,11 +760,11 @@ sub parse {
     my @empty_element_end_tag = ();
     {
         TAG: for my $tag (keys %{$is_empty_element}) {
-            my $start_tag_id = $tracer->symbol_by_name('S_' . $tag);
-            next TAG if not defined $start_tag_id;
-            my $end_tag_id = $tracer->symbol_by_name('E_' . $tag);
-            $empty_element_end_tag[$start_tag_id] = $end_tag_id;
-        }
+	    my $start_tag_id = $tracer->symbol_by_name('S_' . $tag);
+	    next TAG if not defined $start_tag_id;
+	    my $end_tag_id = $tracer->symbol_by_name('E_' . $tag);
+	    $empty_element_end_tag[$start_tag_id] = $end_tag_id;
+	}
     }
 
     my $recce = Marpa::R2::Thin::R->new($thin_grammar);
@@ -771,28 +799,28 @@ sub parse {
     my $empty_element_end_tag;
     RECCE_RESPONSE: while ( $token_number < $token_count ) {
 
-        if ( defined $empty_element_end_tag ) {
-            my $read_result =
-                $recce->alternative( $empty_element_end_tag, RUBY_SLIPPERS_TOKEN,
-                1 );
-            if ( $read_result != $NO_MARPA_ERROR ) {
-                die $thin_grammar->error();
-            }
-            if ($trace_terminals) {
-                say {$trace_fh} 'Virtual end tag accepted: ',
-                    $tracer->symbol_name($empty_element_end_tag)
-                    or Carp::croak("Cannot print: $ERRNO");
-            }
-            if ( $recce->earleme_complete() < 0 ) {
-                die $thin_grammar->error();
-            }
-            my $current_earleme = $recce->current_earleme();
-            die $thin_grammar->error() if not defined $current_earleme;
-            $self->{earleme_to_html_token_ix}->[$current_earleme] =
-                $latest_html_token;
-            $empty_element_end_tag = undef;
-            next RECCE_RESPONSE;
-        } ## end if ( defined $empty_element_end_tag )
+	if ( defined $empty_element_end_tag ) {
+	    my $read_result =
+		$recce->alternative( $empty_element_end_tag, RUBY_SLIPPERS_TOKEN,
+		1 );
+	    if ( $read_result != $NO_MARPA_ERROR ) {
+		die $thin_grammar->error();
+	    }
+	    if ($trace_terminals) {
+		say {$trace_fh} 'Virtual end tag accepted: ',
+		    $tracer->symbol_name($empty_element_end_tag)
+		    or Carp::croak("Cannot print: $ERRNO");
+	    }
+	    if ( $recce->earleme_complete() < 0 ) {
+		die $thin_grammar->error();
+	    }
+	    my $current_earleme = $recce->current_earleme();
+	    die $thin_grammar->error() if not defined $current_earleme;
+	    $self->{earleme_to_html_token_ix}->[$current_earleme] =
+		$latest_html_token;
+	    $empty_element_end_tag = undef;
+	    next RECCE_RESPONSE;
+	} ## end if ( defined $empty_element_end_tag )
 
         my $token = $html_parser_tokens[$token_number];
 
@@ -823,7 +851,7 @@ sub parse {
             $self->{earleme_to_html_token_ix}->[$current_earleme] =
                 $latest_html_token;
 
-            $empty_element_end_tag = $empty_element_end_tag[$attempted_symbol_id];
+	    $empty_element_end_tag = $empty_element_end_tag[$attempted_symbol_id];
             next RECCE_RESPONSE;
         } ## end if ( $read_result != $UNEXPECTED_TOKEN_ID )
 
@@ -897,7 +925,7 @@ sub parse {
             $self->{earleme_to_html_token_ix}->[$current_earleme] =
                 $latest_html_token;
 
-            $empty_element_end_tag = $empty_element_end_tag[$virtual_terminal_to_add];
+	    $empty_element_end_tag = $empty_element_end_tag[$virtual_terminal_to_add];
 
             next RECCE_RESPONSE;
         } ## end if ( defined $virtual_terminal_to_add )
@@ -1195,5 +1223,3 @@ sub Marpa::R2::HTML::html {
 }
 
 1;
-
-# vim: set expandtab shiftwidth=4:
