@@ -1,4 +1,4 @@
-# Copyright 2015 Jeffrey Kegler
+# Copyright 2013 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -21,12 +21,63 @@ use strict;
 use English qw( -no_match_vars );
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.105_000';
+$VERSION        = '2.062000';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
 ## use critic
 
+# Elements of the RECOGNIZER structure
+BEGIN {
+    my $structure = <<'END_OF_STRUCTURE';
+
+    :package=Marpa::R2::Internal::Recognizer
+
+    C { C structure for the recognizer }
+    B_C { C structure for the bocage }
+    O_C { C structure for the Marpa order object }
+    T_C { C structure for the Marpa tree iterator }
+
+    GRAMMAR { the grammar used }
+    FINISHED
+    TOKEN_VALUES
+    SLR { the associated SLR, if there is one }
+
+    TRACE_FILE_HANDLE
+
+    END
+    CLOSURES
+    EVENT_IF_EXPECTED
+    MAX_PARSES
+    RANKING_METHOD
+    TRACE_ACTIONS
+    TRACE_AND_NODES
+    TRACE_BOCAGE
+    TRACE_EARLEY_SETS
+    TRACE_OR_NODES
+    TRACE_TASKS
+    TRACE_TERMINALS
+    TRACE_VALUES
+    TRACE_SL
+    WARNINGS
+
+    { The following fields must be reinitialized when
+    evaluation is reset }
+
+    RULE_RESOLUTIONS
+    SYMBOL_RESOLUTIONS
+    NULL_VALUES
+    EVENTS
+    READ_STRING_ERROR
+
+    { This is the end of the list of fields which
+    must be reinitialized when evaluation is reset }
+
+
+END_OF_STRUCTURE
+    Marpa::R2::offset($structure);
+} ## end BEGIN
+
 package Marpa::R2::Internal::Recognizer;
 
 use English qw( -no_match_vars );
@@ -39,26 +90,14 @@ sub Marpa::R2::Recognizer::new {
     my $recce = bless [], $class;
 
     my $grammar;
-    my $trace_file_handle;
-    for my $arg_hash (@arg_hashes) {
-
-        # Need to capture the trace file handle early
-        my $value;
-        if ( defined( $value = $arg_hash->{trace_file_handle} ) ) {
-            delete $arg_hash->{trace_file_handle};
-            $trace_file_handle = $value;
-        }
-        if ( defined( $value = $arg_hash->{grammar} ) ) {
+    ARG_HASH: for my $arg_hash (@arg_hashes) {
+        if ( defined( $grammar = $arg_hash->{grammar} ) ) {
             delete $arg_hash->{grammar};
-            $grammar = $value;
+            last ARG_HASH;
         }
-    } ## end for my $arg_hash (@arg_hashes)
+    } ## end ARG_HASH: for my $arg_hash (@arg_hashes)
     Marpa::R2::exception('No grammar specified') if not defined $grammar;
 
-    $trace_file_handle //= $grammar->[Marpa::R2::Internal::Grammar::TRACE_FILE_HANDLE] ;
-    local $Marpa::R2::Internal::TRACE_FH =
-        $recce->[Marpa::R2::Internal::Recognizer::TRACE_FILE_HANDLE] = $trace_file_handle;
-
     $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR] = $grammar;
 
     my $grammar_class = ref $grammar;
@@ -78,6 +117,11 @@ sub Marpa::R2::Recognizer::new {
         );
     } ## end if ($problems)
 
+    # set the defaults
+    local $Marpa::R2::Internal::TRACE_FH = my $trace_fh =
+        $recce->[Marpa::R2::Internal::Recognizer::TRACE_FILE_HANDLE] =
+        $grammar->[Marpa::R2::Internal::Grammar::TRACE_FILE_HANDLE];
+
     my $recce_c = $recce->[Marpa::R2::Internal::Recognizer::C] =
         Marpa::R2::Thin::R->new($grammar_c);
     if ( not defined $recce_c ) {
@@ -91,20 +135,16 @@ sub Marpa::R2::Recognizer::new {
 
     $recce_c->ruby_slippers_set(1);
 
-    if (   defined $grammar->[Marpa::R2::Internal::Grammar::ACTION_OBJECT]
-        or defined $grammar->[Marpa::R2::Internal::Grammar::ACTIONS]
-        or not defined $grammar->[Marpa::R2::Internal::Grammar::INTERNAL] )
-    {
-        $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE] =
-            'legacy';
-    } ## end if ( defined $grammar->[...])
-
     ARG_HASH: for my $arg_hash (@arg_hashes) {
         if ( defined( my $value = $arg_hash->{'leo'} ) ) {
             my $boolean = $value ? 1 : 0;
             $recce->use_leo_set($boolean);
             delete $arg_hash->{leo};
         }
+        if ( defined( my $value = $arg_hash->{'_slr'} ) ) {
+            $recce->[Marpa::R2::Internal::Recognizer::SLR] = $value;
+            delete $arg_hash->{_slr};
+        }
     } ## end ARG_HASH: for my $arg_hash (@arg_hashes)
 
     ARG_HASH: for my $arg_hash (@arg_hashes) {
@@ -126,7 +166,6 @@ sub Marpa::R2::Recognizer::new {
     $recce->[Marpa::R2::Internal::Recognizer::WARNINGS]       = 1;
     $recce->[Marpa::R2::Internal::Recognizer::RANKING_METHOD] = 'none';
     $recce->[Marpa::R2::Internal::Recognizer::MAX_PARSES]     = 0;
-    $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS]     = 0;
 
     # Position 0 is not used because 0 indicates an unvalued token.
     # Position 1 is reserved for undef.
@@ -143,14 +182,19 @@ sub Marpa::R2::Recognizer::new {
 
     $recce->set(@arg_hashes);
 
-    if ( $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] > 1 ) {
+    my $trace_terminals =
+        $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] // 0;
+    my $trace_tasks = $recce->[Marpa::R2::Internal::Recognizer::TRACE_TASKS]
+        // 0;
+
+    if ( $trace_terminals > 1 ) {
         my @terminals_expected = @{ $recce->terminals_expected() };
         for my $terminal ( sort @terminals_expected ) {
             say {$Marpa::R2::Internal::TRACE_FH}
                 qq{Expecting "$terminal" at earleme 0}
                 or Marpa::R2::exception("Cannot print: $ERRNO");
         }
-    } ## end if ( $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS...])
+    } ## end if ( $trace_terminals > 1 )
 
     return $recce;
 } ## end sub Marpa::R2::Recognizer::new
@@ -164,44 +208,14 @@ sub Marpa::R2::Recognizer::thin {
     $_[0]->[Marpa::R2::Internal::Recognizer::C];
 }
 
-# For the non-legacy case,
-# I reset the ordering, forcing it to be recalculated
-# for each parse series.
-# But I do not actually allow the ranking method to
-# be changed once a parse is created.
-# Since I am stabilizing Marpa::R2, the "fix" should
-# probably be to save the overhead, rather than
-# to allow 'ranking_method' to be changed.
-# But for now I will do nothing.
-# JK -- Mon Nov 24 17:35:24 PST 2014
 sub Marpa::R2::Recognizer::reset_evaluation {
     my ($recce) = @_;
-    my $grammar = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
-    my $package_source =
-        $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE];
-    if ( defined $package_source and $package_source ne 'legacy' ) {
-
-        # Packaage source, once legacy, stays legacy
-        # Otherwise, reset it
-        $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE] =
-            undef;
-    } ## end if ( defined $package_source and $package_source ne ...)
-    $recce->[Marpa::R2::Internal::Recognizer::NO_PARSE]          = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::ASF_OR_NODES]          = [];
-    $recce->[Marpa::R2::Internal::Recognizer::B_C]                   = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::EVENTS]                = [];
-    $recce->[Marpa::R2::Internal::Recognizer::O_C]                   = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::PER_PARSE_CONSTRUCTOR] = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::READ_STRING_ERROR]     = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE]       = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::NULL_VALUES]        = undef;
-
-    $recce->[Marpa::R2::Internal::Recognizer::REGISTRATIONS]         = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_SYMBOL_ID] = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::CLOSURE_BY_RULE_ID]   = undef;
-
-    $recce->[Marpa::R2::Internal::Recognizer::T_C] = undef;
-    $recce->[Marpa::R2::Internal::Recognizer::TREE_MODE] = undef;
+    $recce->[Marpa::R2::Internal::Recognizer::B_C]              = undef;
+    $recce->[Marpa::R2::Internal::Recognizer::O_C]              = undef;
+    $recce->[Marpa::R2::Internal::Recognizer::T_C]              = undef;
+    $recce->[Marpa::R2::Internal::Recognizer::RULE_RESOLUTIONS] = undef;
+    $recce->[Marpa::R2::Internal::Recognizer::NULL_VALUES]      = undef;
+    $recce->[Marpa::R2::Internal::Recognizer::EVENTS]           = [];
     return;
 } ## end sub Marpa::R2::Recognizer::reset_evaluation
 
@@ -224,38 +238,38 @@ sub Marpa::R2::Recognizer::set {
             );
         } ## end if ( not $ref_type or $ref_type ne 'HASH' )
 
-        state $recognizer_options = {
-            map { ( $_, 1 ) }
-                qw(
-                closures
-                end
-                event_if_expected
-                leo
-                max_parses
-                semantics_package
-                ranking_method
-                too_many_earley_items
-                trace_actions
-                trace_and_nodes
-                trace_bocage
-                trace_earley_sets
-                trace_fh
-                trace_file_handle
-                trace_or_nodes
-                trace_terminals
-                trace_values
-                warnings
-                )
-        };
+	state $recognizer_options = {
+	    map { ( $_, 1 ) }
+		qw(
+		closures
+		end
+		event_if_expected
+		leo
+		max_parses
+		ranking_method
+		too_many_earley_items
+		trace_actions
+		trace_and_nodes
+		trace_bocage
+		trace_earley_sets
+		trace_fh
+		trace_file_handle
+		trace_or_nodes
+		trace_tasks
+		trace_terminals
+		trace_values
+		warnings
+		)
+	};
 
-        if (my @bad_options =
-            grep { not exists $recognizer_options->{$_} }
-            keys %{$args}
-            )
-        {
-            Carp::croak( 'Unknown option(s) for Marpa::R2 Recognizer: ',
-                join q{ }, @bad_options );
-        } ## end if ( my @bad_options = grep { not exists $recognizer_options...})
+	if (my @bad_options =
+	    grep { not exists $recognizer_options->{$_} }
+	    keys %{$args}
+	    )
+	{
+	    Carp::croak( 'Unknown option(s) for Marpa::R2 Recognizer: ',
+		join q{ }, @bad_options );
+	} ## end if ( my @bad_options = grep { not exists $recognizer_options...})
 
         if ( defined( my $value = $args->{'event_if_expected'} ) ) {
             ## It could be allowed, but it is not needed and this is simpler
@@ -272,30 +286,6 @@ sub Marpa::R2::Recognizer::set {
             $recce->[Marpa::R2::Internal::Recognizer::MAX_PARSES] = $value;
         }
 
-        if ( defined( my $value = $args->{'semantics_package'} ) ) {
-
-            # Not allowed once parsing is started
-            if ( defined $recce->[Marpa::R2::Internal::Recognizer::B_C] ) {
-                Marpa::R2::exception(
-                    q{Cannot change 'semantics_package' named argument once parsing has started}
-                );
-            }
-
-            $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE]
-                //= 'semantics_package';
-            if ( $recce
-                ->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE_SOURCE] ne
-                'semantics_package' )
-            {
-                Marpa::R2::exception(
-                    qq{'semantics_package' named argument in conflict with other choices\n},
-                    qq{   Usually this means you tried to use the discouraged 'action_object' named argument as well\n}
-                );
-            } ## end if ( $recce->[...])
-            $recce->[Marpa::R2::Internal::Recognizer::RESOLVE_PACKAGE] =
-                $value;
-        } ## end if ( defined( my $value = $args->{'semantics_package'...}))
-
         if ( defined( my $value = $args->{'ranking_method'} ) ) {
 
             # Not allowed once parsing is started
@@ -368,9 +358,20 @@ sub Marpa::R2::Recognizer::set {
             }
         } ## end if ( defined( my $value = $args->{'trace_or_nodes'} ...))
 
+        if ( defined( my $value = $args->{'trace_tasks'} ) ) {
+            Marpa::R2::exception('trace_tasks must be set to a number >= 0')
+                if $value !~ /\A\d+\z/xms;
+            $recce->[Marpa::R2::Internal::Recognizer::TRACE_TASKS] =
+                $value + 0;
+            if ($value) {
+                say {$trace_fh} "Setting trace_tasks option to $value"
+                    or Marpa::R2::exception("Cannot print: $ERRNO");
+            }
+        } ## end if ( defined( my $value = $args->{'trace_tasks'} ) )
+
         if ( defined( my $value = $args->{'trace_terminals'} ) ) {
             $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] =
-                Scalar::Util::looks_like_number($value) ? $value : 0;
+                $value;
             if ($value) {
                 say {$trace_fh} 'Setting trace_terminals option'
                     or Marpa::R2::exception("Cannot print: $ERRNO");
@@ -401,7 +402,7 @@ sub Marpa::R2::Recognizer::set {
                 Marpa::R2::exception(
                     q{Cannot reset end once evaluation has started});
             }
-            $recce->[Marpa::R2::Internal::Recognizer::END_OF_PARSE] = $value;
+            $recce->[Marpa::R2::Internal::Recognizer::END] = $value;
         } ## end if ( defined( my $value = $args->{'end'} ) )
 
         if ( defined( my $value = $args->{'closures'} ) ) {
@@ -413,8 +414,7 @@ sub Marpa::R2::Recognizer::set {
             }
             my $closures =
                 $recce->[Marpa::R2::Internal::Recognizer::CLOSURES] = $value;
-            for my $action ( keys %{$closures} ) {
-                my $closure = $closures->{$action};
+            while ( my ( $action, $closure ) = each %{$closures} ) {
                 Marpa::R2::exception(qq{Bad closure for action "$action"})
                     if ref $closure ne 'CODE';
             }
@@ -486,6 +486,20 @@ sub Marpa::R2::Recognizer::strip { return 1; }
 
 # Viewing methods, for debugging
 
+BEGIN {
+    my $structure = <<'END_OF_STRUCTURE';
+
+    :package=Marpa::R2::Internal::Progress_Report
+
+    RULE_ID
+    POSITION
+    ORIGIN
+    CURRENT
+
+END_OF_STRUCTURE
+    Marpa::R2::offset($structure);
+} ## end BEGIN
+
 sub Marpa::R2::Recognizer::progress {
     my ( $recce, $ordinal_arg ) = @_;
     my $recce_c = $recce->[Marpa::R2::Internal::Recognizer::C];
@@ -664,9 +678,9 @@ sub Marpa::R2::Recognizer::alternative {
         {
             Marpa::R2::exception('alternative(): value must be undef or ref');
         } ## end if ( $ref_type ne 'SCALAR' and $ref_type ne 'REF' and...)
-        my $value = ${$value_ref};
-        last SET_VALUE_IX if not defined $value;
         $value_ix = scalar @{$token_values};
+	my $value = ${$value_ref};
+	last SET_VALUE_IX if not defined $value;
         push @{$token_values}, $value;
     } ## end SET_VALUE_IX:
     $length //= 1;
@@ -769,12 +783,12 @@ sub Marpa::R2::Recognizer::earleme_complete {
         print {$Marpa::R2::Internal::TRACE_FH} "=== Earley set $latest_set\n"
             or Marpa::R2::exception("Cannot print: $ERRNO");
         print {$Marpa::R2::Internal::TRACE_FH}
-            Marpa::R2::show_earley_set($recce, $latest_set)
+            Marpa::R2::show_earley_set($latest_set)
             or Marpa::R2::exception("Cannot print: $ERRNO");
     } ## end if ( $recce->[Marpa::R2::Internal::Recognizer::TRACE_EARLEY_SETS...])
 
     my $trace_terminals =
-        $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS];
+        $recce->[Marpa::R2::Internal::Recognizer::TRACE_TERMINALS] // 0;
     if ( $trace_terminals > 1 ) {
         my $current_earleme    = $recce_c->current_earleme();
         my $terminals_expected = $recce->terminals_expected();
@@ -845,17 +859,6 @@ sub Marpa::R2::Recognizer::earley_set_size {
     return $recce_c->_marpa_r_earley_set_size($set_id);
 }
 
-sub ahm_describe {
-    my ($recce, $ahm_id)        = @_;
-    my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
-    my $grammar        = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
-    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
-    my $irl_id = $grammar_c->_marpa_g_ahm_irl($ahm_id);
-    my $dot_position = $grammar_c->_marpa_g_ahm_position($ahm_id);
-    if ($dot_position < 0) { return 'R' . $irl_id . q{$} }
-    return 'R' . $irl_id . q{:} . $dot_position;
-}
-
 sub Marpa::R2::show_leo_item {
     my ($recce)        = @_;
     my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
@@ -889,26 +892,24 @@ sub Marpa::R2::show_token_link_choice {
     my ( $recce, $current_earleme ) = @_;
     my $recce_c = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $grammar = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
-    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $tracer  = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
     my $text    = q{};
     my @pieces  = ();
     my ( $token_id, $value_ix ) = $recce_c->_marpa_r_source_token();
-    my $predecessor_ahm = $recce_c->_marpa_r_source_predecessor_state();
+    my $predecessor_state = $recce_c->_marpa_r_source_predecessor_state();
     my $origin_set_id     = $recce_c->_marpa_r_earley_item_origin();
     my $origin_earleme    = $recce_c->earleme($origin_set_id);
     my $middle_earleme    = $origin_earleme;
 
-    if ( defined $predecessor_ahm ) {
+    if ( defined $predecessor_state ) {
         my $middle_set_id = $recce_c->_marpa_r_source_middle();
         $middle_earleme = $recce_c->earleme($middle_set_id);
         push @pieces,
-              'c='
-            . ahm_describe($recce, $predecessor_ahm)
-            . q{@}
+              'p=S'
+            . $predecessor_state . q{@}
             . $origin_earleme . q{-}
             . $middle_earleme;
-    } ## end if ( defined $predecessor_ahm )
+    } ## end if ( defined $predecessor_state )
     my $symbol_name = $tracer->isy_name($token_id);
     push @pieces, 's=' . $symbol_name;
     my $token_length = $current_earleme - $middle_earleme;
@@ -922,10 +923,9 @@ sub Marpa::R2::show_token_link_choice {
 
 # Assumes trace completion source link set by caller
 sub Marpa::R2::show_completion_link_choice {
-    my ( $recce, $link_ahm_id, $current_earleme ) = @_;
+    my ( $recce, $AHFA_state_id, $current_earleme ) = @_;
     my $recce_c = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $grammar = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
-    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $symbols = $grammar->[Marpa::R2::Internal::Grammar::SYMBOLS];
     my $text    = q{};
     my @pieces  = ();
@@ -937,13 +937,14 @@ sub Marpa::R2::show_completion_link_choice {
 
     if ( defined $predecessor_state ) {
         push @pieces,
-              'p='
-            . ahm_describe($recce, $predecessor_state) . q{@}
+              'p=S'
+            . $predecessor_state . q{@}
             . $origin_earleme . q{-}
             . $middle_earleme;
     } ## end if ( defined $predecessor_state )
     push @pieces,
-          'c=' . ahm_describe($recce, $link_ahm_id) . q{@}
+          'c=S'
+        . $AHFA_state_id . q{@}
         . $middle_earleme . q{-}
         . $current_earleme;
     return '[' . ( join '; ', @pieces ) . ']';
@@ -951,10 +952,9 @@ sub Marpa::R2::show_completion_link_choice {
 
 # Assumes trace completion source link set by caller
 sub Marpa::R2::show_leo_link_choice {
-    my ( $recce, $link_ahm_id, $current_earleme ) = @_;
+    my ( $recce, $AHFA_state_id, $current_earleme ) = @_;
     my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $grammar        = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
-    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
     my $symbols        = $grammar->[Marpa::R2::Internal::Grammar::SYMBOLS];
     my $text           = q{};
     my @pieces         = ();
@@ -964,8 +964,8 @@ sub Marpa::R2::show_leo_link_choice {
         $recce_c->_marpa_r_source_leo_transition_symbol();
     push @pieces, 'l=L' . $leo_transition_symbol . q{@} . $middle_earleme;
     push @pieces,
-          'c=' . ahm_describe($recce, $link_ahm_id)
-        . q{@}
+          'c=S'
+        . $AHFA_state_id . q{@}
         . $middle_earleme . q{-}
         . $current_earleme;
     return '[' . ( join '; ', @pieces ) . ']';
@@ -973,29 +973,14 @@ sub Marpa::R2::show_leo_link_choice {
 
 # Assumes trace earley item was set by caller
 sub Marpa::R2::show_earley_item {
-    my ( $recce, $current_es, $item_id ) = @_;
+    my ( $recce, $current_es, $state_id ) = @_;
     my $recce_c        = $recce->[Marpa::R2::Internal::Recognizer::C];
-    my $grammar   = $recce->[Marpa::R2::Internal::Recognizer::GRAMMAR];
-    my $grammar_c = $grammar->[Marpa::R2::Internal::Grammar::C];
-    my $tracer    = $grammar->[Marpa::R2::Internal::Grammar::TRACER];
-
-    my $ahm_id_of_yim = $recce_c->_marpa_r_earley_item_trace($item_id);
-    return if not defined $ahm_id_of_yim;
-
     my $text           = q{};
     my $origin_set_id  = $recce_c->_marpa_r_earley_item_origin();
     my $earleme        = $recce_c->earleme($current_es);
     my $origin_earleme = $recce_c->earleme($origin_set_id);
-    $text .= sprintf "ahm%d: %s@%d-%d", $ahm_id_of_yim,
-        ahm_describe($recce, $ahm_id_of_yim),
-        $origin_earleme, $earleme;
-    my @lines    = $text;
-    my $irl_id = $grammar_c->_marpa_g_ahm_irl($ahm_id_of_yim);
-    my $dot_position = $grammar_c->_marpa_g_ahm_position($ahm_id_of_yim);
-    push @lines, qq{  }
-        . ahm_describe($recce, $ahm_id_of_yim)
-        . q{: }
-        . $tracer->show_dotted_irl($irl_id, $dot_position);
+    $text .= sprintf "S%d@%d-%d", $state_id, $origin_earleme, $earleme;
+    my @pieces    = $text;
     my @sort_data = ();
 
     for (
@@ -1012,7 +997,7 @@ sub Marpa::R2::show_earley_item {
             Marpa::R2::show_token_link_choice( $recce, $earleme )
             ];
     } ## end for ( my $symbol_id = $recce_c->_marpa_r_first_token_link_trace...)
-    my @pieces = map { $_->[-1] } sort {
+    push @pieces, map { $_->[-1] } sort {
                $a->[0] <=> $b->[0]
             || $a->[1] <=> $b->[1]
             || $a->[2] <=> $b->[2]
@@ -1041,45 +1026,50 @@ sub Marpa::R2::show_earley_item {
     } @sort_data;
     @sort_data = ();
     for (
-        my $link_ahm_id = $recce_c->_marpa_r_first_leo_link_trace();
-        defined $link_ahm_id;
-        $link_ahm_id = $recce_c->_marpa_r_next_leo_link_trace()
+        my $AHFA_state_id = $recce_c->_marpa_r_first_leo_link_trace();
+        defined $AHFA_state_id;
+        $AHFA_state_id = $recce_c->_marpa_r_next_leo_link_trace()
         )
     {
         push @sort_data,
             [
             $recce_c->_marpa_r_source_middle(),
-            $link_ahm_id,
+            $AHFA_state_id,
             $recce_c->_marpa_r_source_leo_transition_symbol(),
             Marpa::R2::show_leo_link_choice(
-                $recce, $link_ahm_id, $earleme
+                $recce, $AHFA_state_id, $earleme
             )
             ];
-    } ## end for ( my $link_ahm_id = $recce_c...)
+    } ## end for ( my $AHFA_state_id = $recce_c...)
     push @pieces, map { $_->[-1] } sort {
                $a->[0] <=> $b->[0]
             || $a->[1] <=> $b->[1]
             || $a->[2] <=> $b->[2]
     } @sort_data;
-    push @lines, q{  } . join q{ }, @pieces if @pieces;
-    return join "\n", @lines, q{};
+    return join q{ }, @pieces;
 } ## end sub Marpa::R2::show_earley_item
 
 sub Marpa::R2::show_earley_set {
     my ( $recce, $traced_set_id ) = @_;
     my $recce_c   = $recce->[Marpa::R2::Internal::Recognizer::C];
     my $text      = q{};
-    my @sorted_data = ();
+    my @sort_data = ();
     if ( not defined $recce_c->_marpa_r_earley_set_trace($traced_set_id) ) {
         return $text;
     }
     EARLEY_ITEM: for ( my $item_id = 0;; $item_id++ ) {
-        my $item_desc = Marpa::R2::show_earley_item( $recce, $traced_set_id, $item_id );
-        last EARLEY_ITEM if not defined $item_desc;
-        # We do not sort these any more
-        push @sorted_data, $item_desc;
+        my $state_id = $recce_c->_marpa_r_earley_item_trace($item_id);
+        last EARLEY_ITEM if not defined $state_id;
+        push @sort_data,
+            [
+            $recce_c->_marpa_r_earley_item_origin(), $state_id,
+            Marpa::R2::show_earley_item( $recce, $traced_set_id, $state_id )
+            ];
     } ## end EARLEY_ITEM: for ( my $item_id = 0;; $item_id++ )
-    my @sort_data = ();
+    my @sorted_data =
+        map { $_->[-1] . "\n" }
+        sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] } @sort_data;
+    @sort_data = ();
     POSTDOT_ITEM:
     for (
         my $postdot_symbol_id = $recce_c->_marpa_r_first_postdot_item_trace();
@@ -1115,5 +1105,3 @@ sub Marpa::R2::Recognizer::show_earley_sets {
 } ## end sub Marpa::R2::Recognizer::show_earley_sets
 
 1;
-
-# vim: set expandtab shiftwidth=4:
