@@ -1,4 +1,4 @@
-# Copyright 2015 Jeffrey Kegler
+# Copyright 2013 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.105_000';
+$VERSION        = '2.062000';
 $STRING_VERSION = $VERSION;
 ## no critic(BuiltinFunctions::ProhibitStringyEval)
 $VERSION = eval $VERSION;
@@ -58,6 +58,12 @@ use English qw( -no_match_vars );
 # would conflict with the LHS of the sequence.
 # 
 
+# Undo any rewrite of the symbol name
+sub Marpa::R2::Grammar::original_symbol_name {
+   $_[0] =~ s/\[ prec \d+ \] \z//xms;
+   return shift;
+}
+
 my %node_status =
     map { ; ($_ , q{} ) }
     qw(
@@ -65,7 +71,6 @@ action
 action_name
 adverb_item
 adverb_list
-adverb_list_items
 alternative
 alternatives
 array_descriptor
@@ -82,7 +87,6 @@ lhs
 op_declare
 op_declare_bnf
 parenthesized_rhs_primary_list
-Perl_name
 priorities
 priority_rule
 proper_specification
@@ -99,29 +103,28 @@ single_symbol
 standard_name
 start_rule
 statement
-statements
+statement_body
 symbol
 symbol_name
 );
 
 
 $node_status{'Marpa::R2::Internal::MetaAST'} = q{};
-$node_status{array_descriptor} = "Actions in the form of array descriptors are not allowed";
 $node_status{character_class} = "Character classes are not allowed";
-$node_status{completion_event_declaration} = "Completion events are not allowed";
 $node_status{discard_rule} = ":discard rules are not allowed";
-$node_status{event_specification} = qq{The "event" adverb is not allowed};
-$node_status{latm_specification} = qq{The "latm" adverb is not allowed};
+$node_status{single_quoted_string} = "Quoted strings are not allowed";
 $node_status{lexeme_default_statement} = "The lexeme default statement is not allowed";
 $node_status{lexeme_rule} = "Lexeme statements are not allowed";
+$node_status{completion_event_declaration} = "Completion events are not allowed";
 $node_status{nulled_event_declaration} = "Nulled events are not allowed";
-$node_status{op_declare_match} = "lexical rules are not allowed";
-$node_status{pause_specification} = "The pause adverb is not allowed";
 $node_status{prediction_event_declaration} = "Prediction events are not allowed";
+$node_status{array_descriptor} = "Actions in the form of array descriptors are not allowed";
+$node_status{op_declare_match} = "G0 rules are not allowed";
 $node_status{priority_specification} = "The priority adverb is not allowed";
-$node_status{single_quoted_string} = "Quoted strings are not allowed";
-$node_status{alternative_name} = "Alternative naming is not allowed";
-$node_status{naming} = "Alternative naming is not allowed";
+$node_status{pause_specification} = "The pause adverb is not allowed";
+# 'forgiving' was never documented and may be eliminated
+# $node_status{forgiving_specification} = qq{The "forgiving" adverb is not allowed};
+$node_status{event_specification} = qq{The "event" adverb is not allowed};
 
 my %catch_error_node = 
     map { ; ($_ , 1 ) }
@@ -130,7 +133,7 @@ my %catch_error_node =
 # This code goes to some trouble to report errors with a large enough contet
 # to be meaningful -- rules or alternatives
 
-sub Marpa::R2::Internal::Stuifzand::check_ast_node {
+sub Marpa::R2::Internal::Stuifzand::check_ast {
     my ($node) = @_;
     my $ref_type = ref $node;
     return if not $ref_type;
@@ -147,7 +150,7 @@ sub Marpa::R2::Internal::Stuifzand::check_ast_node {
             last NORMAL_PROCESSING;
         }
         for my $sub_node ( @{$node} ) {
-            $problem = Marpa::R2::Internal::Stuifzand::check_ast_node($sub_node);
+            $problem = Marpa::R2::Internal::Stuifzand::check_ast($sub_node);
             if ($problem) {
                 return $problem if not $catch_error;
                 last NORMAL_PROCESSING;
@@ -169,16 +172,16 @@ sub Marpa::R2::Internal::Stuifzand::check_ast_node {
             $problem_was_here,
             "\n"
         );
-} ## end sub Marpa::R2::Internal::Stuifzand::check_ast_node
+} ## end sub Marpa::R2::Internal::Stuifzand::check_ast
 
 sub parse_rules {
     my ($p_rules_source) = @_;
     my $self             = {};
-    my $ast              = Marpa::R2::Internal::MetaAST->new($p_rules_source);
+    my $parse = {};
+    my $ast              = Marpa::R2::Internal::MetaAST->new($p_rules_source, $parse);
     {
         local $Marpa::R2::Internal::P_SOURCE = $p_rules_source;
-        my $problem = Marpa::R2::Internal::Stuifzand::check_ast_node(
-            $ast->{top_node} );
+        my $problem = Marpa::R2::Internal::Stuifzand::check_ast($ast);
         ## Uncaught problem -- should not happen
         if ($problem) {
             Marpa::R2::exception(
@@ -187,26 +190,8 @@ sub parse_rules {
             );
         } ## end if ($problem)
     }
-    my $hashed_ast = $ast->ast_to_hash();
-    my $start_lhs = $hashed_ast->{'start_lhs'} // $hashed_ast->{'first_lhs'};
-    Marpa::R2::exception( 'No rules in Stuifzand grammar', )
-        if not defined $start_lhs;
-
-    my $internal_start_lhs = '[:start]';
-    $hashed_ast->{'default_g1_start_action'} =
-        $hashed_ast->{'default_adverbs'}->{'G1'}->{'action'};
-    $hashed_ast->{'symbols'}->{'G1'}->{$internal_start_lhs} = {
-        display_form => ':start',
-        description  => 'Internal G1 start symbol'
-    };
-    push @{ $hashed_ast->{rules}->{G1} },
-        {
-        lhs    => $internal_start_lhs,
-        rhs    => [$start_lhs],
-        action => '::first'
-        };
-
-    $self->{rules} = $hashed_ast->{rules}->{G1};
+    my $hashed_ast = $ast->ast_to_hash($parse);
+    $self->{rules} = $hashed_ast->{g1_rules};
     return $self;
 } ## end sub parse_rules
 
