@@ -1,4 +1,4 @@
-# Copyright 2015 Jeffrey Kegler
+# Copyright 2013 Jeffrey Kegler
 # This file is part of Marpa::R2.  Marpa::R2 is free software: you can
 # redistribute it and/or modify it under the terms of the GNU Lesser
 # General Public License as published by the Free Software Foundation,
@@ -20,7 +20,7 @@ use warnings;
 use strict;
 
 use vars qw($VERSION $STRING_VERSION);
-$VERSION        = '2.105_000';
+$VERSION        = '2.062000';
 $STRING_VERSION = $VERSION;
 $VERSION        = eval $VERSION;
 
@@ -88,18 +88,6 @@ sub rule {
     return ($lhs, @rhs);
 }
 
-# Expand a rule into a list of symbol IDs
-sub rule_expand {
-    my ( $self, $rule_id ) = @_;
-    my $grammar     = $self->{g};
-    my $rule_length = $grammar->rule_length($rule_id);
-    return if not defined $rule_length;
-    my $lhs         = ( $grammar->rule_lhs($rule_id) );
-    return ( $lhs,
-        map { $grammar->rule_rhs( $rule_id, $_ ) }
-            ( 0 .. $rule_length - 1 ) );
-} ## end sub rule_expand
-
 sub dotted_rule {
     my ( $self, $rule_id, $dot_position ) = @_;
     my $grammar     = $self->{g};
@@ -147,31 +135,13 @@ sub progress_report {
     return $result;
 } ## end sub progress_report
 
-sub lexer_progress_report {
-    my ( $self, $slr, $ordinal ) = @_;
-    my $thin_slr = $slr->[Marpa::R2::Internal::Scanless::R::C];
-    my $result = q{};
-    $ordinal //= $thin_slr->lexer_latest_earley_set();
-    $thin_slr->lexer_progress_report_start($ordinal);
-    ITEM: while (1) {
-        my ( $rule_id, $dot_position, $origin ) = $thin_slr->lexer_progress_item();
-        last ITEM if not defined $rule_id;
-        $result
-            .= q{@}
-            . $origin . q{: }
-            . $self->dotted_rule( $rule_id, $dot_position ) . "\n";
-    } ## end ITEM: while (1)
-    $thin_slr->lexer_progress_report_finish();
-    return $result;
-} ## end sub progress_report
-
 sub show_dotted_irl {
     my ( $self, $irl_id, $dot_position ) = @_;
     my $grammar_c  = $self->{g};
     my $lhs_id     = $grammar_c->_marpa_g_irl_lhs($irl_id);
     my $irl_length = $grammar_c->_marpa_g_irl_length($irl_id);
 
-    my $text = $self->isy_name($lhs_id) . q{ ::=};
+    my $text = $self->isy_name($lhs_id) . q{ ->};
 
     if ( $dot_position < 0 ) {
         $dot_position = $irl_length;
@@ -179,9 +149,9 @@ sub show_dotted_irl {
 
     my @rhs_names = ();
     for my $ix ( 0 .. $irl_length - 1 ) {
-        my $rhs_nsy_id = $grammar_c->_marpa_g_irl_rhs( $irl_id, $ix );
-        my $rhs_nsy_name = $self->isy_name($rhs_nsy_id);
-        push @rhs_names, $rhs_nsy_name;
+        my $rhs_isy_id = $grammar_c->_marpa_g_irl_rhs( $irl_id, $ix );
+        my $rhs_isy_name = $self->isy_name($rhs_isy_id);
+        push @rhs_names, $rhs_isy_name;
     }
 
     POSITION: for my $position ( 0 .. scalar @rhs_names ) {
@@ -197,12 +167,15 @@ sub show_dotted_irl {
 
 } ## end sub show_dotted_irl
 
-sub show_ahm {
+sub show_AHFA_item {
     my ( $self, $item_id ) = @_;
     my $grammar_c  = $self->{g};
-    my $postdot_id = $grammar_c->_marpa_g_ahm_postdot($item_id);
-    my $text       = "AHM $item_id: ";
+    my $postdot_id = $grammar_c->_marpa_g_AHFA_item_postdot($item_id);
+    my $sort_key   = $grammar_c->_marpa_g_AHFA_item_sort_key($item_id);
+    my $text       = "AHFA item $item_id: ";
     my @properties = ();
+    push @properties, "sort = $sort_key";
+
     if ( $postdot_id < 0 ) {
         push @properties, 'completion';
     }
@@ -212,29 +185,88 @@ sub show_ahm {
     }
     $text .= join q{; }, @properties;
     $text .= "\n" . ( q{ } x 4 );
-    $text .= $self->show_brief_ahm($item_id) . "\n";
+    $text .= $self->show_brief_AHFA_item($item_id) . "\n";
     return $text;
-} ## end sub show_ahm
+} ## end sub show_AHFA_item
 
-sub show_brief_ahm {
+sub show_brief_AHFA_item {
     my ( $self, $item_id ) = @_;
     my $grammar_c  = $self->{g};
-    my $postdot_id = $grammar_c->_marpa_g_ahm_postdot($item_id);
-    my $irl_id     = $grammar_c->_marpa_g_ahm_irl($item_id);
-    my $position   = $grammar_c->_marpa_g_ahm_position($item_id);
+    my $postdot_id = $grammar_c->_marpa_g_AHFA_item_postdot($item_id);
+    my $irl_id     = $grammar_c->_marpa_g_AHFA_item_irl($item_id);
+    my $position   = $grammar_c->_marpa_g_AHFA_item_position($item_id);
     return $self->show_dotted_irl( $irl_id, $position );
-} ## end sub show_brief_ahm
+} ## end sub show_brief_AHFA_item
 
-sub show_ahms {
+sub show_AHFA {
+    my ( $self, $verbose ) = @_;
+    $verbose //= 1;    # legacy is to be verbose, so default to it
+    my $grammar_c        = $self->{g};
+    my $text             = q{};
+    my $AHFA_state_count = $grammar_c->_marpa_g_AHFA_state_count();
+    STATE:
+    for ( my $state_id = 0; $state_id < $AHFA_state_count; $state_id++ ) {
+        $text .= "* S$state_id:";
+        defined $grammar_c->_marpa_g_AHFA_state_leo_lhs_symbol($state_id)
+            and $text .= ' leo-c';
+        $grammar_c->_marpa_g_AHFA_state_is_predict($state_id)
+            and $text .= ' predict';
+        $text .= "\n";
+        my @items = ();
+        for my $item_id ( $grammar_c->_marpa_g_AHFA_state_items($state_id) ) {
+            push @items,
+                [
+                $grammar_c->_marpa_g_AHFA_item_irl($item_id),
+                $grammar_c->_marpa_g_AHFA_item_postdot($item_id),
+                $self->show_brief_AHFA_item($item_id)
+                ];
+        } ## end for my $item_id ( $grammar_c->_marpa_g_AHFA_state_items...)
+        $text .= join "\n", map { $_->[2] }
+            sort { $a->[0] <=> $b->[0] || $a->[1] <=> $b->[1] } @items;
+        $text .= "\n";
+
+        next STATE if not $verbose;
+
+        my @raw_transitions =
+            $grammar_c->_marpa_g_AHFA_state_transitions($state_id);
+        my %transitions = ();
+        while ( my ( $isy_id, $to_state_id ) = splice @raw_transitions, 0, 2 )
+        {
+            my $symbol_name = $self->isy_name($isy_id);
+            $transitions{$symbol_name} = $to_state_id;
+        }
+        for my $transition_symbol ( sort keys %transitions ) {
+            $text .= ' <' . $transition_symbol . '> => ';
+            my $to_state_id = $transitions{$transition_symbol};
+            my @to_descs    = ("S$to_state_id");
+            my $lhs_id =
+                $grammar_c->_marpa_g_AHFA_state_leo_lhs_symbol($to_state_id);
+            if ( defined $lhs_id ) {
+                my $lhs_name = $self->isy_name($lhs_id);
+                push @to_descs, "leo($lhs_name)";
+            }
+            my $empty_transition_state =
+                $grammar_c->_marpa_g_AHFA_state_empty_transition(
+                $to_state_id);
+            $empty_transition_state >= 0
+                and push @to_descs, "S$empty_transition_state";
+            $text .= ( join q{; }, sort @to_descs ) . "\n";
+        } ## end for my $transition_symbol ( sort keys %transitions )
+
+    } ## end STATE: for ( my $state_id = 0; $state_id < $AHFA_state_count...)
+    return $text;
+} ## end sub show_AHFA
+
+sub show_AHFA_items {
     my ($self)    = @_;
     my $grammar_c = $self->{g};
     my $text      = q{};
-    my $count     = $grammar_c->_marpa_g_ahm_count();
+    my $count     = $grammar_c->_marpa_g_AHFA_item_count();
     for my $AHFA_item_id ( 0 .. $count - 1 ) {
-        $text .= $self->show_ahm($AHFA_item_id);
+        $text .= $self->show_AHFA_item($AHFA_item_id);
     }
     return $text;
-} ## end sub show_ahms
+} ## end sub show_AHFA_items
 
 sub isy_name {
     my ( $self, $id ) = @_;
@@ -245,14 +277,14 @@ sub isy_name {
 
     GEN_NAME: {
 
-        if ( $grammar_c->_marpa_g_nsy_is_start($id) ) {
+        if ( $grammar_c->_marpa_g_isy_is_start($id) ) {
             my $source_id = $grammar_c->_marpa_g_source_xsy($id);
             $name = $self->symbol_name($source_id);
             $name .= q<[']>;
             last GEN_NAME;
-        } ## end if ( $grammar_c->_marpa_g_nsy_is_start($id) )
+        } ## end if ( $grammar_c->_marpa_g_isy_is_start($id) )
 
-        my $lhs_xrl = $grammar_c->_marpa_g_nsy_lhs_xrl($id);
+        my $lhs_xrl = $grammar_c->_marpa_g_isy_lhs_xrl($id);
         if ( defined $lhs_xrl and defined $grammar_c->sequence_min($lhs_xrl) )
         {
             my $original_lhs_id = $grammar_c->rule_lhs($lhs_xrl);
@@ -260,7 +292,7 @@ sub isy_name {
             last GEN_NAME;
         } ## end if ( defined $lhs_xrl and defined $grammar_c->sequence_min...)
 
-        my $xrl_offset = $grammar_c->_marpa_g_nsy_xrl_offset($id);
+        my $xrl_offset = $grammar_c->_marpa_g_isy_xrl_offset($id);
         if ($xrl_offset) {
             my $original_lhs_id = $grammar_c->rule_lhs($lhs_xrl);
             $name =
@@ -272,7 +304,7 @@ sub isy_name {
 
         my $source_id = $grammar_c->_marpa_g_source_xsy($id);
         $name = $self->symbol_name($source_id);
-        $name .= '[]' if $grammar_c->_marpa_g_nsy_is_nulling($id);
+        $name .= '[]' if $grammar_c->_marpa_g_isy_is_nulling($id);
 
     } ## end GEN_NAME:
 
